#!/bin/bash

# Check if a directory was provided
if [ -z "$1" ]; then
  echo "Usage: $0 <directory>"
  exit 1
fi

TARGET_DIR="$1"

# Check if the input is a directory
if [ ! -d "$TARGET_DIR" ]; then
  echo "Error: '$TARGET_DIR' is not a directory"
  exit 1
fi

DIR_NAME="$(basename "${TARGET_DIR%/}")"
OUTPUT="ls_filetree_${DIR_NAME}.txt"

echo "Generating file tree with contents from: $TARGET_DIR"
echo "Output will be written to: $OUTPUT"
echo "" >"$OUTPUT"

# Get total number of files for progress bar
TOTAL_FILES=$(find "$TARGET_DIR" -ls | wc -l)
START_TIME=$(date +%s)
COUNT=0
BAR_WIDTH=30
TERM_WIDTH=$(tput cols 2>/dev/null || echo 80)

# Format seconds into HH:MM:SS
format_time() {
  local T=$1
  printf '%02d:%02d:%02d' $((T/3600)) $(((T%3600)/60)) $((T%60))
}

draw_progress() {
  local now=$(date +%s)
  local elapsed=$(( now - START_TIME ))
  local elapsed_fmt=$(format_time $elapsed)
  local percent=$(( COUNT * 100 / TOTAL_FILES ))

  # Cap percentage at 100%
  if [ $percent -gt 100 ]; then
    percent=100
  fi

  local filled=$(( BAR_WIDTH * percent / 100 ))
  local empty=$(( BAR_WIDTH - filled ))

  # Ensure bar fills completely at 100%
  if [ $percent -eq 100 ]; then
    filled=$BAR_WIDTH
    empty=0
  fi

  local size=$(du -h "$OUTPUT" | cut -f1)
  local bar=$(printf "%0.s#" $(seq 1 $filled))
  bar+=$(printf "%0.s-" $(seq 1 $empty))

  local line="Output file: $OUTPUT | Size: $size | $elapsed_fmt | [$bar] ${percent}%"
  
  # Pad line to clear artifacts
  printf "\r%-${TERM_WIDTH}s" "$line"
}

# Use find -ls to get all file information
find "$TARGET_DIR" -ls | while IFS= read -r line; do
  ((COUNT++))
  echo "$line" >>"$OUTPUT"
  draw_progress
done

TOTAL_TIME=$(( $(date +%s) - START_TIME ))
FINAL_SIZE=$(du -h "$OUTPUT" | cut -f1)

draw_progress
echo
echo "Done. Processed $COUNT files in $(format_time $TOTAL_TIME). Final output size: $FINAL_SIZE."
